/**
 * Copyright (c) 2010 Hyfinity Limited. All rights reserved.
 * Scripts for transforming SVG Buttons using WebMaker.
 */
 
var objIconSVG;

/**
 * The SVG file defines some initial size and layout params for the SCG button. 
 * This function uses these to populate the controls that can be used to change the button params.
 */
function initButtonParams()
{
	//Obtain handle for embedded SVG DOM within separate .svg file.
	objIconSVG = document.getElementById("svg_icon").getSVGDocument();
	dijit.byId("button_width").attr("value", objIconSVG.getElementById("button_bg").getAttribute("width"));
	dijit.byId("button_height").attr("value", objIconSVG.getElementById("button_bg").getAttribute("height"));	
	dojo.byId("button_caption").value = objIconSVG.getElementById("button_caption").firstChild.nodeValue;
	dijit.byId("caption_size").attr("value", objIconSVG.getElementById("button_caption").getAttribute("font-size"));
	dijit.byId("button_opacity").attr("value", objIconSVG.getElementById("button_opacity_reflector").getAttribute("opacity"));
}

/**
 * Change the button caption. 
 * @param strCaption New caption for the button.
 */
function changeCaption(strCaption)
{
	objIconSVG.getElementById("button_caption").firstChild.nodeValue = strCaption;	
	centerCaption();
}

/**
 * Change the button color. 
 * @param strColor New color for the button.
 */
function changeButtonColor(strColor)
{
	objIconSVG.getElementById("button_bg").setAttribute("fill", strColor);
	objIconSVG.getElementById("gradient_from_color").setAttribute("stop-color", strColor);	
}

/**
 * Change the width of the button. 
 * @param iWidth New width value for the button.
 */
function changeButtonWidth(iWidth)
{	
	objIconSVG.getElementById("button_bg").setAttribute("width", iWidth);
	objIconSVG.getElementById("button_opacity_reflector").setAttribute("width", iWidth-4);
	objIconSVG.getElementById("button_opacity_mask").setAttribute("width", iWidth-4);
	centerCaption();

}

/**
 * Change the height of the button. 
 * @param iHeight New height value for the button.
 */
function changeButtonHeight(iHeight)
{	
	objIconSVG.getElementById("button_bg").setAttribute("height", iHeight);
	objIconSVG.getElementById("button_opacity_reflector").setAttribute("height", iHeight/2);
	objIconSVG.getElementById("button_opacity_mask").setAttribute("height", iHeight/2);
	centerCaption();	
}

/**
 * Change the color of the button caption. 
 * @param strColor New color for the button caption.
 */
function changeCaptionColor(strColor)
{
	objIconSVG.getElementById("button_caption").setAttribute("fill", strColor);	
}

/**
 * Change the font size for the button caption. 
 * @param iSize New font size for the button caption.
 */
function changeCaptionSize(iSize)
{
	objIconSVG.getElementById("button_caption").setAttribute("font-size", iSize);	
	centerCaption();
}

/**
 * Change the opacity for the glass reflector on the button. 
 * @param iOpacity New opacity value for the glass reflector.
 */
function changeButtonOpacity(iOpacity)
{	
	objIconSVG.getElementById("gradient_to_color").setAttribute("stop-opacity", iOpacity);
	objIconSVG.getElementById("gradient_from_color").setAttribute("stop-opacity", iOpacity);
	objIconSVG.getElementById("button_opacity_reflector").setAttribute("opacity", iOpacity);
}

/**
 * Whenever the button or the caption size is changed the caption needs to repositioned within the center of the button.
 */
function centerCaption()
{
	var buttonBBox = objIconSVG.getElementById("button_bg").getBBox();	
	var objCaption = objIconSVG.getElementById("button_caption");
	var captionBBox = objCaption.getBBox();		
	var iCaptionX = buttonBBox.x + buttonBBox.width/2 - captionBBox.width/2;
	var iCaptionY = buttonBBox.y + buttonBBox.height/2 + captionBBox.height/2 - (buttonBBox.height*0.05);
	objCaption.setAttribute("x", iCaptionX);
	objCaption.setAttribute("y", iCaptionY);
}

/**
 * Add the event handlers for the dojo controls within the WebMaker form.
 */
function initDojoEvents()
{			
	dojo.connect(dijit.byId("button_width"),"onChange",function()
	{ 		
		changeButtonWidth(this.attr("value"))
	});
	dojo.connect(dijit.byId("button_height"),"onChange",function()
	{ 		
		changeButtonHeight(this.attr("value"))
	});
	dojo.connect(dijit.byId("button_color"),"onChange",function()
	{ 		
		changeButtonColor(this.attr("value"))
	});
	dojo.connect(dijit.byId("caption_color"),"onChange",function()
	{ 		
		changeCaptionColor(this.attr("value"))
	});
	dojo.connect(dijit.byId("caption_size"),"onChange",function()
	{ 		
		changeCaptionSize(this.attr("value"))
	});
	dojo.connect(dijit.byId("button_opacity"),"onChange",function()
	{ 		
		changeButtonOpacity(this.attr("value"))
	});
	
}
dojo.addOnLoad(initDojoEvents);

 